from IPython.core.magic import Magics, magics_class, line_cell_magic
import yapf
import codecs

@magics_class
class Pep8Magics(Magics):

    @line_cell_magic
    def pep8(self, line, cell=None):
        """
        pep8 IPython magic extension, cf. https://pypi.python.org/pypi/pep8

        Magic methods:
            %%pep8 
            < python code ... >
    
            %pep8 filename
        """
        style = """{
            BASED_ON_STYLE = pep8
            COLUMN_LIMIT = 132
        }"""
        source = cell if cell else codecs.open(line, "r", "utf8").read()
        result = yapf.yapf_api.FormatCode(source, style_config=style)[0]
        get_ipython().set_next_input(result.rstrip(), replace = True)



from IPython.core.magic import (Magics, magics_class, line_cell_magic)
import dis

@magics_class
class DisMagics(Magics):
    @line_cell_magic
    def dis(self, line, cell = None):
        """
        Disassembling CPython code magic extension.
        This is a thin wrapper around dis.dis().
        Cf. https://docs.python.org/3/library/dis.html

        Magic methods:
            %dis statements

            %%dis
            statements
        """
        print(dis.dis(line if line else cell))



from .truth import TruthMagics
from .pep8 import Pep8Magics
from .tutor import TutorMagics
from .dis import DisMagics

def load_ipython_extension(ipython):
    """Load the extension in IPython."""
    ipython.register_magics(Pep8Magics)
    ipython.register_magics(TruthMagics)
    ipython.register_magics(TutorMagics)
    ipython.register_magics(DisMagics)



from IPython.core.magic import (Magics, magics_class, line_cell_magic)
from itertools import product
import re

def truth(*tests, **kwargs):
    bools = kwargs.get("bools", ("False", "True"))
    width = max(len(bools[0]), len(bools[1]))
    (off, tests) = zip(*[(test.startswith("?"),test.lstrip("?").strip()) for test in tests])
    names = sorted(set(m.group(0) for m in re.finditer(r"\b\w\b", " ".join(tests))))
    labels = [name.ljust(width) for name in names] + list(tests)
    result = ["| %s |" % " | ".join(labels)]
    result.append("|-%s-|" % ("-|-".join("-" * len(label) for label in labels)))
    for values in product((0, 1), repeat = len(names)):
        cells = [bools[value].ljust(width) for value in values]
        for (i, test) in enumerate(tests):
            if off[i]:
                cells.append("".ljust(len(labels[i + len(names)])))
            else:
                for (name, value) in zip(names, values):
                    test = re.sub(r"\b%s\b" % name, str(value), test)
                cells.append(bools[eval(test)].ljust(len(labels[i + len(names)])))
        result.append("| %s |" % " | ".join(cells))
    return "\n".join(result)

@magics_class
class TruthMagics(Magics):
    @line_cell_magic
    def truth(self, line, cell = None, bools = ("False", "True")):
        """
        Truth Table Generator IPython magic extension

        Magic methods:
            %truth bool_expr

            %truth? bool_expr

            %truth01 bool_expr

            %truth bool_expr_1, bool_expr_2, ..., bool_expr_n

            %truth bool_expr_1, ? bool_expr_2, ..., bool_expr_n

            %%truth
            bool_expr_1
            bool_expr_2
            ...
            bool_expr_n

            %%truth
            bool_expr_1
            ? bool_expr_2
            ...
            bool_expr_n
        """
        if line:
            print(truth(*line.split(", "), bools = bools))
        elif cell:
            print(truth(*cell.strip().split("\n"), bools = bools))
    
    @line_cell_magic
    def truth01(self, line, cell = None):
        self.truth(line, cell = cell, bools = "01")
    
    @line_cell_magic
    def truth10(self, line, cell = None):
        self.truth(line, cell = cell, bools = "01")
    
    @line_cell_magic
    def truthFV(self, line, cell = None):
        self.truth(line, cell = cell, bools = "FV")
    
    @line_cell_magic
    def truthVF(self, line, cell = None):
        self.truth(line, cell = cell, bools = "FV")
    
    @line_cell_magic
    def truthFT(self, line, cell = None):
        self.truth(line, cell = cell, bools = "FT")
    
    @line_cell_magic
    def truthTF(self, line, cell = None):
        self.truth(line, cell = cell, bools = "FT")


if __name__ == "__main__":
    print(truth("(a and b) or (b and c) or (not a and c)", bools = "01"))



from setuptools import setup, find_packages
from codecs import open
import algo_magic

long_description = """\
This set of IPython magic extensions is provided to the first year students enrolled in the algorithmics course at ISFATES (University of Lorraine).

Installation
------------

The recommended way to install the algo_magic extension is to use pip:

::

    pip install algo_magic

If this fails, ensure first you have a working Python 3 installation.


Usage
-----

Load the magic extensions:

::

    %load_ext algo_magic


More info
---------

`Source code on GitHub
<https://github.com/laowantong/algo_magic/>`_
"""

with open("README.rst", "w", "utf8") as f:
    f.write(long_description)

setup(
    name="algo_magic",
    version=algo_magic.__version__,
    license="MIT",
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Education',
        'Intended Audience :: Information Technology',
        'Topic :: Database',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5',
        'Framework :: IPython'
    ],
    keywords='algorithmics tools',
    zip_safe=False,
    description=("Set of IPython magic extensions used in the first year algorithmics course at University of Lorraine."),
    url='https://github.com/laowantong/algo_magic',
    author='Aristide Grange',
    author_email='name.surname@univ-lorraine.fr',
    packages=find_packages(exclude=[]),
    install_requires=["ipython", "yapf"],
    long_description=long_description
)



from IPython.core.magic import Magics, magics_class, line_cell_magic
from IPython.display import HTML, display
import urllib
import re

@magics_class
class TutorMagics(Magics):

    @line_cell_magic
    def tutor(self, line, cell=None):
        """
        Python Tutor IPython magic extension

        Magic methods:
            %%tutor 
            < python code ... >
    
            %tutor filename

            %%tutor 640x400
            < python code ... >
    
            %tutor 640x400 filename
        """
        (width, height, filename) = (None, None, None)
        source = cell
        m = re.match(r" *(?:(\d+).(\d+))? *(.*)", line)
        if m:
            (width, height, filename) = m.groups()
            if filename:
                source = open(filename).read()
            else:
                source = cell
        if not width or not height:
            (width, height) = (800, 500)
        code = urllib.parse.urlencode({"code": source})
        display(HTML('<iframe width="%s" height="%s" frameborder="0" src="http://pythontutor.com/iframe-embed.html#%s&py=3"></iframe>' % (width, height, code)))
